// idt_usb_hid.cpp : implementation file
//
#include "stdafx.h"
#include "idt_usb_hid.h"

CIdtUsbHid::CIdtUsbHid()
{
	m_uiVID =0;
	m_uiPID =0;
	m_ReadEvent =0;
	m_hidDevice =INVALID_HANDLE_VALUE;
	
	m_ReadEvent = CreateEvent(NULL, TRUE, TRUE, NULL);
	//Set the members of the overlapped structure.
    m_overlapped.hEvent = m_ReadEvent;
    m_overlapped.Offset = 0;
    m_overlapped.OffsetHigh = 0;
}

unsigned int CIdtUsbHid::LookDevice(UINT auiVid, UINT auiPid) 
{
	GUID			HidGuid;	    //UID of HID device class
	HDEVINFO		hDevInfoSet;	//handle to all hid device's info set.
	SP_DEVICE_INTERFACE_DATA	devIfData;  //All HID devices interface data.
	SP_DEVICE_INTERFACE_DETAIL_DATA *pIfDetail; //interface detail
	HIDD_ATTRIBUTES		attrs;		//Structrue with VID and PID
	ULONG			len, required;	//length of interface datail structure
	PHIDP_PREPARSED_DATA	preparsedData;

	m_uiVID =auiVid;
	m_uiPID =auiPid;
	
	if(m_hidDevice !=INVALID_HANDLE_VALUE)
	{
		CloseHandle(m_hidDevice);
		m_hidDevice =INVALID_HANDLE_VALUE;
	}
	
	HidD_GetHidGuid(&HidGuid);
	if( (hDevInfoSet=SetupDiGetClassDevs(&HidGuid, NULL, NULL,
		DIGCF_PRESENT|DIGCF_INTERFACEDEVICE)) == INVALID_HANDLE_VALUE)
	{
		return 0;
	}
	devIfData.cbSize = sizeof(devIfData);
	int index=0;
	while( SetupDiEnumDeviceInterfaces(hDevInfoSet, 0, 
		&HidGuid, index++, &devIfData) != 0)
	{   
        //this call should fail, just get the detail structrue length
		SetupDiGetDeviceInterfaceDetail( hDevInfoSet, &devIfData, NULL, 0, &len, NULL);	
		
		pIfDetail = (SP_DEVICE_INTERFACE_DETAIL_DATA*)malloc(len);
		pIfDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
		if( ! SetupDiGetDeviceInterfaceDetail(hDevInfoSet, &devIfData, pIfDetail, len, &required, NULL))
		{
			SetupDiDestroyDeviceInfoList(hDevInfoSet);
			free(pIfDetail);
			return 0;
		}
		m_hidDevice=CreateFile(pIfDetail->DevicePath, GENERIC_WRITE|GENERIC_READ, 
			FILE_SHARE_READ|FILE_SHARE_WRITE,(LPSECURITY_ATTRIBUTES) NULL,
			OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
		if(m_hidDevice==INVALID_HANDLE_VALUE)
		{
			DWORD luiError =GetLastError();
			free(pIfDetail);
			continue;//return 0;
		}
		memset(m_devPathName, 0 , 1024);
		strcpy(m_devPathName, pIfDetail->DevicePath);
		free(pIfDetail);
		if( !HidD_GetAttributes(m_hidDevice, &attrs))
		{
			CloseHandle(m_hidDevice);
			m_hidDevice = INVALID_HANDLE_VALUE;		
			continue;//return 0;
		}
		if( attrs.VendorID!=m_uiVID || attrs.ProductID!=m_uiPID)
		{
			CloseHandle(m_hidDevice);
			m_hidDevice = INVALID_HANDLE_VALUE;			
			continue;
		}
		
		if( ! HidD_GetPreparsedData(m_hidDevice, &preparsedData))
		{
			CloseHandle(m_hidDevice);
			m_hidDevice = INVALID_HANDLE_VALUE;
			SetupDiDestroyDeviceInfoList(hDevInfoSet);			
			return 0;
		}

		if( ! HidP_GetCaps(preparsedData, &m_caps))
		{
			CloseHandle(m_hidDevice);
			m_hidDevice = INVALID_HANDLE_VALUE;
			HidD_FreePreparsedData(preparsedData);
			SetupDiDestroyDeviceInfoList(hDevInfoSet);			
			return 0;
		}
		HidD_FreePreparsedData(preparsedData);
		SetupDiDestroyDeviceInfoList(hDevInfoSet);

		return 1;	//OK, to here, we find one right device, and get it's caps.
	}
	SetupDiDestroyDeviceInfoList(hDevInfoSet);
    
	return 0;
}

unsigned int CIdtUsbHid::Write(unsigned char* aucmpData, unsigned int auiDataLen) 
{
	unsigned char lucmCommand[68];
	unsigned int  luiCommandLen =auiDataLen;
	unsigned int  luiCommandCount =1 + auiDataLen/63;
	if((auiDataLen % 63) ==0x00) luiCommandCount --;
	unsigned int  luii;
	DWORD dwBytesWritten;
	lucmCommand[0] =0x00;

	do{
		for(luii=0;luii<68;luii++)
		{
			lucmCommand[luii] =0x00;
		}

		if(luiCommandCount >1)
		{
			luiCommandLen -=63;
			lucmCommand[1] =0xBF;//63;
			for(luii=2;luii<65;luii++)
			{
				lucmCommand[luii] =*aucmpData++;
			}
		}
		else
		{
			lucmCommand[1] =luiCommandLen;
			for(luii=2;luii<65;luii++)
			{
				lucmCommand[luii] =*aucmpData++;
			}
		}

		m_overlapped.Offset =0;
		m_overlapped.OffsetHigh =0;
		if(!WriteFile(m_hidDevice, lucmCommand, m_caps.OutputReportByteLength, &dwBytesWritten, &m_overlapped))
		{
			DWORD dwErrorCode =GetLastError();
			if(dwErrorCode == ERROR_IO_PENDING)
			{
				if(WAIT_OBJECT_0 ==WaitForSingleObject(m_ReadEvent, 1000))
				{		
					ResetEvent(m_ReadEvent);
				}
				else
				{
					return 0;
				}
			}
			else
			{
				return 0;
			}
		}
		luiCommandCount--;
	}while(luiCommandCount);

	return 1;
}

unsigned int CIdtUsbHid::GetReport(unsigned char* aucmpData, unsigned int auiWaitInMs)
{

	BYTE lucmResponse[1024];
	unsigned int  luiResponseLen =0;
	DWORD dwBytesRead;

	aucmpData[0] =0x00;

	if(!ReadFile(m_hidDevice, lucmResponse, m_caps.InputReportByteLength, &dwBytesRead, &m_overlapped))
	{
		DWORD hres;
		DWORD errCode = GetLastError();
		switch(errCode)
		{
			case ERROR_IO_PENDING:			
				hres=WaitForSingleObject(m_ReadEvent, auiWaitInMs);
				if(WAIT_OBJECT_0 == hres)
				{		
					if(GetOverlappedResult(m_hidDevice, &m_overlapped, &dwBytesRead, FALSE))
					{
						ResetEvent(m_ReadEvent);
					}
				}
				else
				{
//					//Time out........
					if(!CancelIo(m_hidDevice))
					{
						ResetEvent(m_ReadEvent);
						CloseHandle(m_hidDevice);
						m_hidDevice =INVALID_HANDLE_VALUE;
						return 0;
					}
				}
				break;

			case ERROR_DEVICE_NOT_CONNECTED:
				CloseHandle(m_hidDevice);
				m_hidDevice =INVALID_HANDLE_VALUE;
				return 0;

			default:
				CloseHandle(m_hidDevice);
				m_hidDevice =INVALID_HANDLE_VALUE;
				return 0;
		}
	}
	
	if(dwBytesRead)
	{

		for(unsigned int luii=0; luii< dwBytesRead; luii++)
				aucmpData[luii] = lucmResponse[luii];			
				
		return dwBytesRead;		// read real data.

	}
	else
	{
		aucmpData[0] =0x00;
		return 0;
	}

}

unsigned int CIdtUsbHid::GetResponse(unsigned char* aucmpData, unsigned int auiRespLen, unsigned int auiWaitInMs) 
{
	unsigned char lucmPartResponse[68];
	unsigned int  luiResponseLen =0;
	unsigned int  luii;
	DWORD dwBytesRead;

	while(1)
	{
		GetReport(lucmPartResponse, auiWaitInMs);
		
		if(lucmPartResponse[0] ==0x00) break;
		
		dwBytesRead =lucmPartResponse[0] & 0x7F;

		for(luii =0; (luii< dwBytesRead) && (luiResponseLen < auiRespLen); luii++)
		{
			aucmpData[luiResponseLen++] =lucmPartResponse[luii+1];
		}

		if(lucmPartResponse[0] < 0x80) break;
	}
	
	return luiResponseLen;
}

unsigned int CIdtUsbHid::SendCommand(UINT auiVid, UINT auiPid, unsigned char* aucmpSource, unsigned int auiSourceLen, unsigned char* aucmpResp, unsigned int auiRespLen, unsigned int auiWaitInMs)
{
	if(m_hidDevice ==INVALID_HANDLE_VALUE)
	{
		LookDevice(auiVid, auiPid);
	}

	if(m_hidDevice !=INVALID_HANDLE_VALUE)
	{
		if(auiSourceLen)
		{
			if(!Write(aucmpSource, auiSourceLen))
			{
				LookDevice(auiVid, auiPid);
			}
		}

		if(aucmpResp && auiRespLen)
		{
			if(m_hidDevice ==INVALID_HANDLE_VALUE)
			{
				LookDevice(auiVid, auiPid);
			}
			
			if(m_hidDevice !=INVALID_HANDLE_VALUE)
			{
				UINT luiResult =GetResponse(aucmpResp, auiRespLen, auiWaitInMs);
				if(!luiResult)
				{
					LookDevice(auiVid, auiPid);
					if(m_hidDevice !=INVALID_HANDLE_VALUE)
					{
						return GetResponse(aucmpResp, auiRespLen, auiWaitInMs);
					}
				}
				else
				{
					return luiResult;
				}
			}
		}
	}
	//
	return 0;
}


void CIdtUsbHid::CloseDevice() 
{
	CloseHandle(m_hidDevice);
	m_hidDevice =INVALID_HANDLE_VALUE;
}

